//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "NNTPMods.h"
#include "openssl.h"
#include "qsadbs.h"
#include "sha256.h"
#include "esub.h"
#include "sampledlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "OverbyteIcsNntpCli"
#pragma link "OverbyteIcsWndControl"
#pragma link "OverbyteIcsWSocket"
#pragma resource "*.dfm"
//TNNTPMod *NNTPMod;
int NNTPThrdHandle = NULL;
TNNTPThread *NNTPThrd = NULL;
HANDLE NNTPModHandle = NULL;
HANDLE MainFormHandle = NULL;
//---------------------------------------------------------------------------
// TNNTPThread
//---------------------------------------------------------------------------

void __fastcall GetNNTPMail(TForm *Owner, const AnsiString &account, bool subsonly, bool catchup)
{
   if (NNTPThrd)
	  return;
   NNTPThrd = new TNNTPThread(Owner, account, subsonly, catchup);
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::TNNTPThread(bool CreateSuspended)
   : TThread(false)
{
   NNTPThrdHandle = Handle;
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::TNNTPThread(TForm *Owner, const AnsiString &account,
   bool subsonly, bool catchup)
   : TThread(false)
{
   MainFormHandle = Owner->Handle;
   NNTPMod = new TNNTPMod(Owner);
   NNTPModHandle = NNTPMod->Handle;
   NNTPMod->subsonly = subsonly;
   NNTPMod->catchup = catchup;
   NNTPMod->account = account;
   NNTPThrdHandle = Handle;
   DB->LoadAccount(account, NNTPMod->profile);
   //FreeOnTerminate = true;
}
//---------------------------------------------------------------------------

__fastcall TNNTPThread::~TNNTPThread()
{
   if (NNTPMod->NNTP->State != nntpNotConnected)
	  NNTPMod->NNTP->Abort();
   Snooze(100);

   NNTPModHandle = NULL;
   delete NNTPMod;
   Snooze(100);
   NNTPMod = NULL;
   NNTPThrdHandle = NULL;
   NNTPThrd = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPThread::Execute()
{
   NNTPMod->GetTheMail();
}

//---------------------------------------------------------------------------
// NNTPMod
//---------------------------------------------------------------------------

/*void __fastcall GetNNTPMail(TForm *Owner, const AnsiString &account, bool subsonly, bool catchup)
{
   if (NNTPMod)
	  return;
   NNTPMod = new TNNTPMod(Owner);
   NNTPModHandle = NNTPMod->Handle;
   NNTPMod->subsonly = subsonly;
   NNTPMod->catchup = catchup;
   NNTPMod->account = account;
   NNTPMod->StartRetrieve();
}*/
//---------------------------------------------------------------------------

__fastcall TNNTPMod::TNNTPMod(TComponent* Owner)
	: TForm(Owner)
{
   transcript = new TStringList();
   proxy = new TStringList();
   subjects = new TStringList();
   profile = new TStringList();
   message = new TStringList();
   TIniFile *ini = new TIniFile(AppIni());
   newsgroup = ini->ReadString("settings", "newsgroup", "alt.anonymous.messages");
   nntpproxy = ini->ReadString("settings", "nntpproxy", "none");
   maxloglines = ini->ReadInteger("settings", "loglines", 1000);
   SSLContext->SslCipherList = ini->ReadString(SSLCipherSuites, NntpSSLciphersuite, NntpSSLCipherSuite);
   delete ini;
   data = new TStringList();
   msgman = new TMsgManager(data);
   ShowProgress(0,0,"");
}
//---------------------------------------------------------------------------

__fastcall TNNTPMod::~TNNTPMod()
{
   if (NNTP->State != nntpNotConnected)
	  NNTP->Abort();

   DB->SaveAccount(account, profile);
   SaveLog();

   ShowStatus("");
   if (!lasterror && !timeout)
	  ShowProgress(0,0,"");
   delete data;
   delete message;
   delete profile;
   delete msgman;
   delete subjects;
   delete proxy;
   delete transcript;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::SaveLog()
{
   TStringList *lines = new TStringList();
   DB->GetText("log", lines);
   lines->Add("");
   lines->AddStrings(transcript);
   while (lines->Count > maxloglines)
	  lines->Delete(0);
   DB->PutText("log", lines);
   delete lines;
}
//---------------------------------------------------------------------------

char* __fastcall AStr2CStr(const AnsiString &astring)
{
   int len = astring.Length();
   char *cstr = new char[len+1];
   strncpy(cstr, astring.c_str(), len);
   cstr[len] = '\0';
   return cstr;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::OwnerPerform(UINT msg, WPARAM wparam, LPARAM lparam)
{
   PostMessage(MainFormHandle, msg, wparam, lparam);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::OwnerErrBox(const AnsiString &Text, TMsgBoxTypes Type, UINT Buttons)
{
   AnsiString msg = Text +"|" + IntToStr(Type) + "|" + IntToStr(Buttons);
   //int len = msg.Length();
   //char *cstr = new char[len+1];
   //strncpy(cstr, msg.c_str(), len); cstr[len] = '\0';
   //char *cstr = AStr2CStr(msg);
   OwnerPerform(WM_MESSAGEBOX, 0, (LPARAM)AStr2CStr(msg));
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::ShowProgress(const double current, const double total, const AnsiString &article)
{
   AnsiString msg;
   if (current && total)
   {
	  msg = IntToStr((int)current)+" of "+IntToStr((int)total);
	  if (mails)
		 msg += "  ("+IntToStr(mails)+")";
   }
   //int len = msg.Length();
   //char *cstr = new char[len+1];
   //strncpy(cstr, msg.c_str(), len); cstr[len] = '\0';
   WORD Pcnt = (WORD)PercentComplete(current);
   //char *cstr = AStr2CStr(msg);
   OwnerPerform(WM_SHOWPROGRESS, (WORD)Pcnt, (LPARAM)AStr2CStr(msg));
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::ShowStatus(const AnsiString &msg)
{
   //int len = msg.Length();
   //char *cstr = new char[len+1];
   //strncpy(cstr, msg.c_str(), len); cstr[len] = '\0';
   //char *cstr = AStr2CStr(msg);
   OwnerPerform(WM_SHOWSTATUS, 0 , (LPARAM)AStr2CStr(msg));
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::Display(const AnsiString &msg)
{
   //int len = msg.Length();
   //char *cstr = new char[len+1];
   //strncpy(cstr, msg.c_str(), len); cstr[len] = '\0';
   //char *cstr = AStr2CStr(msg);
   OwnerPerform(WM_LOGDISPLAY, 0, (LPARAM)AStr2CStr(msg));
   transcript->Add(msg);
   if (msg.Pos("Connect") == 1)
	  ShowStatus(msg);
   else
	  ShowStatus("");
   Snooze(10);
}
//---------------------------------------------------------------------------

int __fastcall TNNTPMod::PercentComplete(double next)
{
   if (totalarticles > 0)
	  return (int)((next / totalarticles) * 100);
   return (int)next;
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod::SetupProxy()
{
   NNTP->CtrlSocket->SocksServer = "";
   NNTP->CtrlSocket->SocksPort = "";
   NNTP->CtrlSocket->SocksLevel = "4a";
   NNTP->CtrlSocket->SocksUsercode = "";
   NNTP->CtrlSocket->SocksPassword = "";
   NNTP->CtrlSocket->SocksAuthentication = socksNoAuthentication;
   assert(nntpproxy != "");

   if (nntpproxy != "none")
   {
	  TStringList *proxies = new TStringList();
	  DB->GetText("proxies", proxies);
	  proxy->CommaText = proxies->Values[nntpproxy];
	  delete proxies;
	  if (proxy->Text.IsEmpty())
		 return false;

	  assert(proxy->Count > 0);
	  assert(proxy->Values["host"] != "");
	  assert(proxy->Values["port"] != "");
	  assert(proxy->Values["socks"] != "" && proxy->Values["socks"] != "no");
	  NNTP->CtrlSocket->SocksServer = proxy->Values["host"];
	  NNTP->CtrlSocket->SocksPort = proxy->Values["port"];
	  NNTP->CtrlSocket->SocksLevel = proxy->Values["socks"];

	  if (ReadBool(proxy, "auth", false))
	  {
		 assert(proxy->Values["password"] != "");
		 assert(proxy->Values["userid"] != "");
		 NNTP->CtrlSocket->SocksPassword = proxy->Values["password"];
		 NNTP->CtrlSocket->SocksUsercode = proxy->Values["userid"];
		 NNTP->CtrlSocket->SocksAuthentication = socksAuthenticateUsercode;
	  }
   }
   return true;
}
//--------------------------------------------------------------------------

bool __fastcall TNNTPMod::GetTheMail()
{
   if (NNTP->State != nntpNotConnected)
	  NNTP->Abort();

   if (ReadString(profile, "key", "").IsEmpty())
   {
	  OwnerErrBox("No NNTP account defined!    ", MT_INFO, MB_OK);
	  return false;
   }

   server = ReadString(profile, "host", "");
   if (server.IsEmpty())
   {
	  OwnerErrBox("No NNTP server defined!    ", MT_INFO, MB_OK);
	  return false;
   }

   if (!SetupProxy())
   {
	  OwnerErrBox("The proxy, "+nntpproxy+", doesn't exist!   ", MT_ERROR, MB_OK);
	  return false;
   }

   OwnerPerform(WM_LOGDISPLAY, CLEARLOG, 0);
   ShowStatus(subsonly ? "checking..." : "preparing...");
   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());
   Display(datetime);
   Display("Proxy: " + GetTok(0, nntpproxy, ','));
   Display("Account: " + ReadString(profile, "key", "error"));
   Display("Connecting to " + server);

   NNTP->Host = server;
   NNTP->Port = ReadString(profile, "port", "119");
   NNTP->UserName = ReadString(profile, "userid", "");
   NNTP->PassWord = ReadString(profile, "password", "");
   AnsiString ssltype = ReadString(profile, "tls", "none").LowerCase();
   NNTP->CtrlSocket->SslEnable = ssltype == "tls" || ssltype == "ssl";

   curmsg = ReadInteger(profile, "lastarticle", -1);
   if (curmsg > 0)
	  curmsg++;

   totalarticles = 0;
   canceled = false;
   aborted = false;
   timeout = false;
   mails = 0;

   // get subjects to compare -- eliminate inactive subjects
   DB->LoadSubjects(subjects);
   int x = 0; while (x < subjects->Count)
   {
	  if (!subjects->Strings[x].Pos("active=1"))
		 subjects->Delete(x);
	  else
		 x++;
   }

   connecting = true;
   ShowProgress(0,0,"");
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
 //TimeoutTimer->Enabled = true;
   NNTP->Connect();
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod::SubjectMatch(const AnsiString &msgsubject)
{
   bool ok;
   AnsiString line, ephrase, cmpsubject;
   assert(msgsubject != "");

   TStringList *subdata = new TStringList();
   for (int i = 0; i < subjects->Count; i++)
   {
	  subdata->CommaText = subjects->Strings[i],
	  assert(subdata->Count > 0);
	  if (subdata->Values["active"] != "1")
		 continue;
	  cmpsubject = subdata->Values["subject"];
	  assert(cmpsubject != "");

	  if (subdata->Values["esub"] == "1")
	  {
		 ephrase = subdata->Values["ephrase"];
		 assert(ephrase != "");
		 ok = IsSubjectBLOWeSub(msgsubject, cmpsubject, ephrase);
		 if (!ok)
			ok = IsSubjectIDEAeSub(msgsubject, cmpsubject, ephrase);
		 if (ok)
			break;
	  }
	  else
	  {
		 ok = TestSubjectHSUB(msgsubject, cmpsubject);
		 if (ok)
			break;
	  }
   }

   delete subdata;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TNNTPMod::SubjectMatch(TMsgManager *msg)
{
   AnsiString cmpsubject, ephrase, kphrase, subno,
	  msgsubject = msg->GetHeader("Subject:");
   assert(msgsubject != "");

   bool ok;
   TStringList *subdata = new TStringList();
   for (int i = 0; i < subjects->Count; i++)
   {
	  subdata->CommaText = subjects->Strings[i],
	  assert(subdata->Count > 0);
	  if (subdata->Values["active"] != "1")
		 continue;

	  cmpsubject = subdata->Values["subject"];
	  assert(cmpsubject != "");

	  if (subdata->Values["esub"] == "1")
	  {
		 ephrase = subdata->Values["ephrase"];
		 assert(ephrase != "");
		 ok = IsSubjectBLOWeSub(msgsubject, cmpsubject, ephrase);
		 if (!ok)
			ok = IsSubjectIDEAeSub(msgsubject, cmpsubject, ephrase);
		 if (ok)
		 {
			msg->Lines->Insert(0, "Subject-No: "+subdata->Values["subno"]);
			break;
		 }
	  }
	  else
	  {
		 ok = TestSubjectHSUB(msgsubject, cmpsubject);
		 if (ok)
		 {
			msg->Lines->Insert(0, "Subject-No: "+subdata->Values["subno"]);
			break;
		 }
	  }
   }
   delete subdata;
   return ok;
}
//-------------------------------------------------------------------------

void __fastcall TNNTPMod::RetrieveComplete()
{
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTOP, 0); // just in case it hasn't

   if (NNTP->State != nntpNotConnected)
	  NNTP->Abort();
   while (NNTP->State != nntpNotConnected)
	  Snooze(50);

   if (!subsonly && !nada)
   {
	  if (curmsg < 0)
		 curmsg = 1;

	  if (!timeout)
		 Display("Ending message: " + IntToStr(curmsg-1));

	  profile->Values["lastarticle"] = curmsg-1;
	  DB->SaveAccount(account, profile);
   }
   if (timeout)
   {
	  Display(ConnectionClosed);
	  Display("Failed!");
   }
   else
   {
	  if (mails && !catchup)
		 Display(IntToStr(mails)+AnsiString(" new message")+(mails>1?"s":"")+
			(subsonly ? " waiting" : ""));
	  else
		 Display("No mail");
   }

   OwnerPerform(WM_UPDATECOMPLETE, canceled, catchup ? 0 : mails);
   OwnerPerform(WM_DELNNTPTHREAD, 0, 0);
   OwnerPerform(WM_SAVEDATABASE, 0, 0);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::WMNNTPCancel(TMessage &Msg)
{
   canceled = true;
   Display("User canceled operation!");
   RetrieveComplete();
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::WMNNTPTimeout(TMessage &Msg)
{
   if (Msg.WParam != TIMERTIMEOUT)
	  return;
   timeout = true;
   Display("OPERATION TIMED OUT!");
   RetrieveComplete();
}
//---------------------------------------------------------------------------
//  NNTP Client
//---------------------------------------------------------------------------

bool displaying = true;
void __fastcall TNNTPMod::NNTPDisplay(TObject *Sender, Pointer MsgBuf,
	  int MsgLen)
{
   AnsiString line = (char*)MsgBuf;
   if (displaying)
   {
	  if (line.Pos("200 "+server) != 1)
		 transcript->Add(line);
	  if (line.Pos("> article") == line.Length() - 8)
		 displaying = false;
   }
   else
   {
	  if (line == ".")
	  {
		 displaying = true;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPMessageBegin(TObject *Sender)
{
   data->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPMessageLine(TObject *Sender)
{
   data->Add(NNTP->LastResponse);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPMessageEnd(TObject *Sender)
{
   //AnsiString msgsubject = msgman->GetHeader("Subject:");
   try
   {
	  if (SubjectMatch(msgman))
	  {
		 data->Insert(0, curmsg);
		 //char *cstr = AStr2CStr(data->Text);
		 //int len = data->Text.Length();
		 //char *cstr = new char[len+1];
		 //strncpy(cstr, data->Text.c_str(), len); cstr[len] = '\0';
		 OwnerPerform(WM_SAVEMESSAGE, ++mails, (LPARAM)AStr2CStr(data->Text));
	  }
   }
   catch(EConvertError &e) // converterror = not a hashed or esubed subject
   {                       // ignore this
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPRequestDone(TObject *Sender, TNntpRequest RqType,
	  WORD ErrCode)
{
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTOP, 0);
 //TimeoutTimer->Enabled = false;
   AnsiString name,msg,subject,ssltype;
   if (ErrCode && ErrCode != 421 && ErrCode != 423 && ErrCode != 400)
   {
	  if (lasterror)
		 return;

	  lasterror = ErrCode;
	  AnsiString errmsg = NNTP->ErrorMessage;
	  if (errmsg.IsEmpty() && ErrCode == 411)
		 errmsg = "No such newsgroup!";
	  if (errmsg.IsEmpty())
		 errmsg = WSocketErrorDesc(ErrCode);
	  errmsg = "RequestDone Error: " + IntToStr(ErrCode) + " " + errmsg;
	  Display(errmsg);

	  //char *cstr = AStr2CStr(transcript->Text);
	  OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(transcript->Text));
	  RetrieveComplete();
	  return;
   }

   switch(RqType)
   {
	  case nntpConnect:
		 while (NNTP->State != nntpReady)
			Snooze(10);
		 OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
		 //TimeoutTimer->Enabled = true;
		 if (NNTP->UserName.IsEmpty())
			NNTP->Group(newsgroup);
		 else
			NNTP->Authenticate();
		 return;
	  case nntpAuthenticate:
		 while (NNTP->State != nntpReady)
			Snooze(10);
		 OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
		 //TimeoutTimer->Enabled = true;
		 NNTP->Group(newsgroup);
		 return;
	  case nntpGroup:
		 ShowStatus("");
		 while (NNTP->State != nntpReady)
			Snooze(10);
		 Display("Newsgroup: "+newsgroup+" "+IntToStr(NNTP->ArticleFirst)+" "+
			IntToStr(NNTP->ArticleLast));
		 if (catchup)
		 {
			curmsg = NNTP->ArticleLast + 1;
			Display("New starting message: " + IntToStr(curmsg));
			profile->Values["lastarticle"] = curmsg - 1;
		  //OwnerPerform(WM_SAVEPROFILE, 0, 0);
			DB->SaveAccount(account, profile);
			RetrieveComplete();
			return;
		 }
		 if (curmsg < 0)
		 {
			int howmany = HowManyMessages((TForm*)Owner, NNTP->ArticleLast - NNTP->ArticleFirst);
			if (howmany < 0)
			{
			   nada = true;
			   RetrieveComplete();
			   return;
			}
			curmsg = NNTP->ArticleLast - howmany;
		 }
		 if (curmsg < NNTP->ArticleFirst)
			curmsg = NNTP->ArticleFirst;
		 Display("Starting message: " + IntToStr(curmsg));
		 Snooze(10);
		 if (curmsg > NNTP->ArticleLast)
		 {
			Display("Nothing to do!");
			nada = true;
			RetrieveComplete();
			return;
		 }
		 totalarticles = NNTP->ArticleLast - curmsg;
		 next = 0;
		 OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
		 //TimeoutTimer->Enabled = true;
		 if (subsonly)
			NNTP->XHdr(NULL, "subject", IntToStr(curmsg)+"-");
		 else
			NNTP->ArticleByNumber(curmsg, NULL);
		 return;
	  case nntpArticleByNumber:
		 while (NNTP->State != nntpReady)
			Snooze(10);
		 data->Clear();
		 curmsg++;
		 next++;
		 if (curmsg <= NNTP->ArticleLast)
		 {
			OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTART, 0);
			//TimeoutTimer->Enabled = true;
			ShowProgress(next, totalarticles, curmsg);
			NNTP->ArticleByNumber(curmsg, NULL);
			return;
		 }
		 RetrieveComplete();
		 return;
	  case nntpXHdr:
		 RetrieveComplete();
		 return;
	  case nntpAbort:
		 //Display("Operation aborted!");
		 aborted = true;
		 return;
	  case nntpQuit:
		 //while (NNTP->State != nntpReady)
		 //	Snooze(10);
		 RetrieveComplete();
		 return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPSessionClosed(TObject *Sender, WORD ErrCode)
{
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTOP, 0);
 //TimeoutTimer->Enabled = false;
   transcript->Add(NNTP->LastResponse);
   catchup = false;

   if (ErrCode)
   {
	  lasterror = ErrCode;
	  AnsiString errmsg = NNTP->ErrorMessage;
	  if (errmsg.IsEmpty())
		 errmsg = WSocketErrorDesc(ErrCode);
	  errmsg = "SessionClosed Error: " + IntToStr(ErrCode) + " " + errmsg;
	  transcript->Add(errmsg);
	  Display(errmsg);
	  Snooze(10);
	  if (lasterror)
		 return;

	  Display("Connection closed!");
	  //char *cstr = AStr2CStr(transcript->Text);
	  OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(transcript->Text));
	  RetrieveComplete();
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPSessionConnected(TObject *Sender, WORD ErrCode)
{
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTOP, 0);
 //TimeoutTimer->Enabled = false;
   transcript->Add(NNTP->LastResponse);
   if (ErrCode)
   {
	  lasterror = ErrCode;
	  AnsiString errmsg = NNTP->ErrorMessage;
	  if (errmsg.IsEmpty())
		 errmsg = WSocketErrorDesc(ErrCode);
	  errmsg = "SessionConnect Error: "+IntToStr(ErrCode)+errmsg;
	  Display(errmsg);
	  //char *cstr = AStr2CStr(transcript->Text);
	  OwnerPerform(WM_PROGRESSERROR, 0, (LPARAM)AStr2CStr(transcript->Text));
	  RetrieveComplete();
	  return;
   }
   connecting = false;
   working = true;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPXHdrBegin(TObject *Sender)
{
   OwnerPerform(WM_NNTPTIMEOUTTIMER, TIMERSTOP, 0);
 //TimeoutTimer->Enabled = false;
   data->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPXHdrEnd(TObject *Sender)
{
   for (int i = 0; i < data->Count; i++)
   {
	  AnsiString artno = GetTok(0, data->Strings[i], ' ');
	  ShowProgress(i, data->Count, (mails ? IntToStr(mails)+" new": AnsiString("")));
	  try
	  {
		 AnsiString msgsubject = GetTok(1, data->Strings[i], ' ');
		 if (SubjectMatch(msgsubject))
		 {
			mails++;
		 }
	  }
	  catch(EConvertError &e)
	  {
	  }
   }
   ShowStatus("");
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPXHdrLine(TObject *Sender)
{
   data->Add(NNTP->LastResponse);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPRcvdData(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TNNTPMod::NNTPSendData(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------
