/*
*  keyringdlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "keyringdlgs.h"
#include "gnupg.h"
#include "global.h"
#include "utility.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TKeyringDlg *KeyringDlg;
//---------------------------------------------------------------------------

bool __fastcall ManagePGPKeyrings(TForm *Owner)
{
   bool ok = false;
   TKeyringDlg *keydlg = new TKeyringDlg(Owner);
   keydlg->ShowModal();
   if (keydlg->ModalResult == mrOk)
   {
	  if (keydlg->Secring->Modified || keydlg->Pubring->Modified)
		 ok = true;
   }
   delete keydlg;
   return ok;
}

//---------------------------------------------------------------------------

__fastcall TKeyringDlg::TKeyringDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Secring->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   Pubring->Font->Assign(Secring->Font);
   delete ini;

   Secring->Clear();
   Pubring->Clear();
   if (FileExists(GPGConf()))
   {
	  TStringList *lines = new TStringList();
	  lines->LoadFromFile(GPGConf());
	  for (int i = 0; i < lines->Count; i++)
	  {
		 AnsiString line = lines->Strings[i];
		 if (line.LowerCase().TrimLeft().Pos("secret-keyring ") == 1)
			Secring->Text = line.SubString(16, line.Length());
		 if (line.LowerCase().TrimLeft().Pos("keyring ") == 1)
			Pubring->Text = line.SubString(9, line.Length());
	  }
	  delete lines;
   }
   Secring->Modified = false;
   Pubring->Modified = false;
   OkBtn->Enabled = FileExists(Secring->Text) && FileExists(Pubring->Text);

   bool qsl = FileExists(AppDir()+"qsl.exe");
   bool qs  = FileExists(AppDir()+"qs.exe");
   if (qsl || qs)
   {
	  Secring->ParentColor = Color;
	  Pubring->ParentColor = Color;
	  Secring->Enabled = false;
	  Pubring->Enabled = false;
	  Button1->Enabled = false;
	  Button2->Enabled = false;
	  if (qsl && !qs || qsl && qs)
		 Label4->Caption = "NOTICE: Make your keyring changes from your QSL program.";
	  if (!qsl && qs)
		 Label4->Caption = "NOTICE: Make your keyring changes from your QS program.";
	  Label4->Show();
	  CancelBtn->Enabled = false;
	  readonly = true;
	  OkBtn->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Secring->Modified || Pubring->Modified)
   {
	  if (ModalResult == mrCancel)
	  {
		 switch (MsgBox(this,
		   "Your keyrings have changed. Save you changes?   ",
			MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL: CanClose = false;
			case IDNO: return;
		 }
	  }
	  SaveConf();
   }
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::SaveConf()
{
   TStringList *lines = new TStringList();
   if (!FileExists(GPGConf()))
   {
	  lines->Add("# QS maintains the values for keyring, secret-keyring,");
	  lines->Add("# and default-key. If you add or modify values ABOVE these");
	  lines->Add("# entries, do so with extreme care. Items added below these");
	  lines->Add("# will be deleted.");
	  lines->Add("no-greeting");
	  lines->Add("no-mdc-warning");
	  lines->Add("no-auto-check-trustdb");
	  lines->Add("trust-model always");
	  lines->Add("no-default-keyring");
   }
   else
   {
	  lines->LoadFromFile(GPGConf());
   }

   bool secadded = false, pubadded = false, ideadded = false;
   for (int i = 0; i < lines->Count; i++)
   {
	  AnsiString line = lines->Strings[i].TrimLeft().LowerCase();
	  if (!secadded && line.Pos("secret-keyring ") == 1)
	  {
		 lines->Strings[i] = "secret-keyring " + GetShortName(Secring->Text);
		 secadded = true;
	  }
	  if (!pubadded && line.Pos("keyring ") == 1)
	  {
		 lines->Strings[i] = "keyring " + GetShortName(Pubring->Text);
		 pubadded = true;
	  }
	  if (!ideadded && line.Pos("load-extension ") == 1 && line.Pos("idea.dll") > 0)
	  {
		 lines->Strings[i] = "load-extension " + GPGDir() + "idea.dll";
		 ideadded = true;
	  }
   }

   if (!ideadded)
	  lines->Add("load-extension " + GPGDir() + "idea.dll");
   if (!secadded)
	  lines->Add("secret-keyring " + GetShortName(Secring->Text));
   if (!pubadded)
	  lines->Add("keyring " + GetShortName(Pubring->Text));

   lines->SaveToFile(GPGConf());
   delete lines;
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::Button1Click(TObject *Sender)
{
   AnsiString pubring = Pubring->Text;
   if (!pubring.IsEmpty() && FileExists(pubring))
	  OpenSecringDialog->InitialDir = ExtractFilePath(pubring);

   if (OpenSecringDialog->Execute())
   {
	  Secring->Text = OpenSecringDialog->FileName;
	  Secring->Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::Button2Click(TObject *Sender)
{
   AnsiString secring = Secring->Text;
   if (!secring.IsEmpty() && FileExists(secring))
	  OpenPubringDialog->InitialDir = ExtractFilePath(secring);

   if (OpenPubringDialog->Execute())
   {
	  Pubring->Text = OpenPubringDialog->FileName;
	  Pubring->Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::KeyringChange(TObject *Sender)
{
   OkBtn->Enabled = FileExists(Secring->Text) && FileExists(Pubring->Text);
}
//---------------------------------------------------------------------------

void __fastcall TKeyringDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/keyrings.htm>keyshelp");
}
//---------------------------------------------------------------------------

