/*
*  nntpmandlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "nntpmandlgs.h"
#include "nntphostdlgs.h"
#include "global.h"
#include "utility.h"
#include "msgmans.h"
#include "qsadbs.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TNNTPManDlg *NNTPManDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageNNTPHosts(TForm *Owner)
{
   TNNTPManDlg *nntpdlg = new TNNTPManDlg(Owner);
   nntpdlg->ShowModal();
   bool ok = nntpdlg->ModalResult == mrOk;
   delete nntpdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TNNTPManDlg::TNNTPManDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "nntpmandlg", ListView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height) +
	  IntToStr(ListView->Columns->Items[0]->Width) +
	  IntToStr(ListView->Columns->Items[1]->Width) +
	  IntToStr(ListView->Columns->Items[2]->Width);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   defhost = ini->ReadString("settings", "account", "none");
   delete ini;

   hosts = new TStringList();
   DB->LoadAccounts(hosts);
   TStringList *cfg = new TStringList();

   for (int i = 0; i < hosts->Count; i++)
   {
	  cfg->CommaText = hosts->Values[hosts->Names[i]];
	  TListItem *item = ListView->Items->Add();
	  item->Caption = cfg->Values["key"];
	  item->SubItems->Add(cfg->Values["host"]);
	  item->SubItems->Add(cfg->Values["userid"]);
	  item->SubItems->Add(cfg->Values["port"]);
	  item->SubItems->Add(cfg->Values["auth"]);
	  item->SubItems->Add(cfg->Values["password"]);
	  item->SubItems->Add(cfg->Values["tls"]);
	  item->SubItems->Add(cfg->Values["lastarticle"]);
   }

   LoadServerBox();
   modified = false;
}
//---------------------------------------------------------------------------

__fastcall TNNTPManDlg::~TNNTPManDlg()
{
   if (IntToStr(Width)+IntToStr(Height)+
	   IntToStr(ListView->Column[0]->Width)+
	   IntToStr(ListView->Column[1]->Width)+
	   IntToStr(ListView->Column[2]->Width) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "nntpmandlg", ListView, ini);
	  delete ini;
   }
   delete hosts;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
   if (modified)
   {
	  if (ModalResult == mrCancel)
	  {
		 switch (MsgBox(this,
			"Your SMTP hosts info is modified. Save your changes?   ",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL:
			   CanClose = false;
			case IDNO:
			   return;
		 }
		 ModalResult = mrOk;
	  }

	  hosts->Clear();
	  TStringList *host = new TStringList();
	  for (int i = 0; i < ListView->Items->Count; i++)
	  {
		 host->Clear();
		 TListItem *item = ListView->Items->Item[i];
		 assert(item->SubItems->Count == 7);

		 host->Values["key"] = item->Caption;
		 host->Values["host"] = item->SubItems->Strings[0];
		 host->Values["userid"] = item->SubItems->Strings[1];
		 host->Values["port"] = item->SubItems->Strings[2];
		 host->Values["auth"] = item->SubItems->Strings[3];
		 host->Values["password"] = item->SubItems->Strings[4];
		 host->Values["tls"] = item->SubItems->Strings[5];
		 host->Values["lastarticle"] = item->SubItems->Strings[6];
		 hosts->Add(item->Caption + "=" + host->CommaText);
	  }
	  DB->SaveAccounts(hosts);
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::PopupMenu1Popup(TObject *Sender)
{
   Edit1->Enabled = ListView->Selected;
   Delete1->Enabled = Edit1->Enabled;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::Delete1Click(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   if (MsgBox(this,
	  "Delete the smtp host: " + ListView->Selected->Caption + "?   ",
	  MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  ListView->Selected->Delete();
	  modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::Add1Click(TObject *Sender)
{
   TStringList *host = new TStringList();
   TStringList *existing = new TStringList();
   for (int i = 0; i < ListView->Items->Count; i++)
	  existing->Add(ListView->Items->Item[i]->Caption);

   if (AddNNTPHost(this, host, existing))
   {
	  TListItem *item = ListView->Items->Add();
	  item->Caption = host->Values["key"];
	  item->SubItems->Add(host->Values["host"]);
	  item->SubItems->Add(host->Values["userid"]);
	  item->SubItems->Add(host->Values["port"]);
	  item->SubItems->Add(host->Values["auth"]);
	  item->SubItems->Add(host->Values["password"]);
	  item->SubItems->Add(host->Values["tls"]);
	  item->SubItems->Add(host->Values["lastarticle"]);
	  modified = true;
   }
   delete existing;
   delete host;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::Edit1Click(TObject *Sender)
{
   if (!ListView->Selected)
	  return;

   TStringList *host = new TStringList();
   TStringList *existing = new TStringList();

   for (int i = 0; i < ListView->Items->Count; i++)
   {
	  TListItem *item = ListView->Items->Item[i];
	  if (item == ListView->Selected)
	  {
		 assert(item->SubItems->Count == 7);
		 host->Values["key"] = item->Caption;
		 host->Values["host"] = item->SubItems->Strings[0];
		 host->Values["userid"] = item->SubItems->Strings[1];
		 host->Values["port"] = item->SubItems->Strings[2];
		 host->Values["auth"] = item->SubItems->Strings[3];
		 host->Values["password"] = item->SubItems->Strings[4];
		 host->Values["tls"] = item->SubItems->Strings[5];
		 host->Values["lastarticle"] = item->SubItems->Strings[6];
	  }
	  else
		 existing->Add(item->Caption);
   }
   if (EditNNTPHost(this, host, existing))
   {
	  TListItem *item = ListView->Selected;
	  AnsiString oldkey = item->Caption, newkey = host->Values["key"];
	  if (newkey != oldkey && newkey != HostBox->Text)
	  {
		 int idx = HostBox->Items->IndexOf(oldkey);
		 assert(idx >= 0);
		 HostBox->ItemIndex = -1;
		 Snooze(100);
		 HostBox->Items->Strings[idx] = newkey;
		 HostBox->ItemIndex = idx;
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteString("settings", "account", newkey);
		 delete ini;
		 AnsiString accnames = oldkey + '\255' + newkey;
	   ((TForm*)Owner)->Perform(WM_ACCOUNTCHANGE, 0, (LPARAM)accnames.c_str());
	  }
	  item->Caption = newkey;
	  item->SubItems->Strings[0] = host->Values["host"];
	  item->SubItems->Strings[1] = host->Values["userid"];
	  item->SubItems->Strings[2] = host->Values["port"];
	  item->SubItems->Strings[3] = host->Values["auth"];
	  item->SubItems->Strings[4] = host->Values["password"];
	  item->SubItems->Strings[5] = host->Values["tls"];
	  item->SubItems->Strings[6] = host->Values["lastarticle"];
	  modified = true;
   }
   delete existing;
   delete host;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::ListViewClick(TObject *Sender)
{
   EditBtn->Enabled = ListView->Selected;
   DeleteBtn->Enabled = EditBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::HostBoxDropDown(TObject *Sender)
{
   TComboBox *HostBox = dynamic_cast<TComboBox*>(Sender);
   if (!HostBox) return;

   HostBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::HostBoxCloseUp(TObject *Sender)
{
   TComboBox *HostBox = dynamic_cast<TComboBox*>(Sender);
   if (!HostBox) return;

   if (HostBox->Text == "none")
	 HostBox->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::HostBoxSelect(TObject *Sender)
{
   TComboBox *HostBox = dynamic_cast<TComboBox*>(Sender);
   if (!HostBox) return;

   if (HostBox->Text == "none")
	 HostBox->Font->Style = TFontStyles() << fsBold;
   else
	 HostBox->Font->Style = TFontStyles();
   defhost = HostBox->Text;

   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "account", HostBox->Text);
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::HostBoxChange(TObject *Sender)
{
   /*assert(HostBox->Text != "");
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "account", HostBox->Text);
   defhost = HostBox->Text;
   modified = true;
   delete ini;*/
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::LoadServerBox()
{
   HostBox->Items->Clear();
   HostBox->Sorted = true;
   for (int i = 0; i < hosts->Count; i++)
   {
	  HostBox->Items->Add(hosts->Names[i]);
   }
   HostBox->Sorted = false;

   HostBox->Items->Insert(0, "none");
   HostBox->ItemIndex = HostBox->Items->IndexOf(defhost);
   CheckReportDefaultServerMissing();

   HostBoxSelect(HostBox);
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::CheckReportDefaultServerMissing()
{
   if (HostBox->ItemIndex < 0)
   {
	  MsgBox(this,
		 "The NNTP host you've been using, " + defhost + ", is missing! You  \n"
		 "don't currently have a host assigned.",
		 MT_WARN, MB_OK);
	  HostBox->ItemIndex = 0;
	  defhost = "none";
	  TIniFile *ini = new TIniFile(AppIni());
	  ini->WriteString("settings", "account", defhost);
	  delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNNTPManDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/nntpman.htm>newshelp");
}
//---------------------------------------------------------------------------

