/*
*  optionsdlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "optionsdlgs.h"
#include "global.h"
#include "utility.h"
#include "pgpcache.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TOptionsDlg *OptionsDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageProgramOptions(TForm *Owner)
{
   TOptionsDlg *opsdlg = new TOptionsDlg(Owner);
   opsdlg->ShowModal();
   bool ok = opsdlg->ModalResult == mrOk && opsdlg->Modified();
   delete opsdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TOptionsDlg::TOptionsDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   newdlgfont = orgdlgfont = ini->ReadString("settings", "dialogfont", OptsDefDialogFont);
   AssignFont(DlgFont->Font, orgdlgfont);
   newwinfont = orgwinfont = ini->ReadString("settings", "windowfont", OptsDefWindowFont);
   AssignFont(WinFont->Font, orgwinfont);
   newedifont = orgedifont = ini->ReadString("settings", "editorfont", OptsDefEditorFont);
   AssignFont(EdiFont->Font, orgedifont);
   orghideprxy = ini->ReadBool("settings", "hideprxysel", false);
   HidePrxyBox->Checked = orghideprxy;
   orghideacct = ini->ReadBool("settings", "hideacctsel", false);
   HideAcctBox->Checked = orghideacct;
   orgusetray = ini->ReadBool("settings", "usetray", false);
   UseTrayBox->Checked = orgusetray;
   orgcheckmail = ini->ReadBool("settings", "checkmail", false);
   CheckMailBox->Checked = orgcheckmail;
   CheckMailBoxClick(NULL);
   orgnotify = ini->ReadBool("settings", "notifyqsl", true);
   Notify->Checked = orgnotify;
   orgcheckmins = (short)ini->ReadInteger("settings", "checkmailmins", 30);
   UpDown1->Position = orgcheckmins;
   orgmaxlines = (short)ini->ReadInteger("settings", "loglines", 1000);
   UpDown3->Position = orgmaxlines;
   orgcachemins = (short)ini->ReadInteger("settings", "cachemins", 0);
   UpDown2->Position = orgcachemins;
   orgsavedecrypt = ini->ReadBool("settings", "savedecrypted", false);
   SaveDecrypted->Checked = orgsavedecrypt;
   orgmaskbox = ini->ReadBool("settings", "maskgpgversion", false);
   MaskBox->Checked = orgmaskbox;
   orgtimeout = ini->ReadInteger("settings", "nntptimeout", 30);
   NNTPTimeOut->Position =(short) orgtimeout;
   delete ini;

   ClearBtn->Enabled = PGPCache->Count();
   Font->Assign(DlgFont->Font);
   EdiFont->Caption = EdiFont->Font->Name + "  " + IntToStr(EdiFont->Font->Size) + "pt";
   DlgFont->Caption = DlgFont->Font->Name + "  " + IntToStr(DlgFont->Font->Size) + "pt";
   WinFont->Caption = WinFont->Font->Name + "  " + IntToStr(WinFont->Font->Size) + "pt";
   PageControl->TabIndex = 0;
}
//---------------------------------------------------------------------------

__fastcall TOptionsDlg::~TOptionsDlg()
{
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Modified())
   {
	  if (ModalResult == mrCancel)
	  {
		 switch(MsgBox(this,
			"Your settings are modified. Save your changes?   ",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL: CanClose = false;
			case IDNO: return;
		 }
		 ModalResult = mrOk;
	  }

	  if (Modified())
	  {
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteBool("settings", "usetray", UseTrayBox->Checked);
		 ini->WriteBool("settings", "checkmail", CheckMailBox->Checked);
		 ini->WriteBool("settings", "savedecrypted", SaveDecrypted->Checked);
		 ini->WriteBool("settings", "notifyqsl", Notify->Checked);
		 ini->WriteBool("settings", "hideprxysel", HidePrxyBox->Checked);
		 ini->WriteBool("settings", "hideacctsel", HideAcctBox->Checked);
		 ini->WriteString("settings", "dialogfont", newdlgfont);
		 ini->WriteString("settings", "windowfont", newwinfont);
		 ini->WriteString("settings", "editorfont", newedifont);
		 ini->WriteInteger("settings", "checkmailmins", UpDown1->Position);
		 ini->WriteInteger("settings", "loglines", UpDown3->Position);
		 ini->WriteInteger("settings", "cachemins", UpDown2->Position);
		 ini->WriteBool("settings", "maskgpgversion", MaskBox->Checked);
		 ini->WriteInteger("settings", "nntptimeout", NNTPTimeOut->Position);
		 delete ini;
	  }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::Modified()
{
   return UseTrayBox->Checked != orgusetray ||
	  CheckMailBox->Checked != orgcheckmail ||
	  SaveDecrypted->Checked != orgsavedecrypt ||
	  UpDown1->Position != orgcheckmins ||
	  UpDown3->Position != IntToStr(orgmaxlines) ||
	  newdlgfont != orgdlgfont ||
	  newedifont != orgedifont ||
	  newwinfont != orgwinfont ||
	  HidePrxyBox->Checked != orghideprxy ||
	  HideAcctBox->Checked != orghideacct ||
	  UpDown2->Position != orgcachemins ||
	  Notify->Checked != orgnotify ||
	  MaskBox->Checked != orgmaskbox ||
	  NNTPTimeOut->Position != orgtimeout;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::DialogFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(DlgFont->Font);
   if (FontDialog->Execute())
   {
	  DlgFont->Font->Assign(FontDialog->Font);
	  DlgFont->Caption = Font->Name + "  " +
		 IntToStr(Font->Size) + "pt";
	  newdlgfont = GetFontStr(FontDialog->Font);
	  Font->Assign(FontDialog->Font);
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::WindowFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(WinFont->Font);
   if (FontDialog->Execute())
   {
	  WinFont->Font->Assign(FontDialog->Font);
	  WinFont->Caption = WinFont->Font->Name + "  " +
		 IntToStr(WinFont->Font->Size) + "pt";
	  newwinfont = GetFontStr(WinFont->Font);
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::EditorFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(EdiFont->Font);
   if (FontDialog->Execute())
   {
	  EdiFont->Font->Assign(FontDialog->Font);
	  EdiFont->Caption = EdiFont->Font->Name + "  " +
		 IntToStr(EdiFont->Font->Size) + "pt";
	  newedifont = GetFontStr(EdiFont->Font);
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::CheckMailBoxClick(TObject *Sender)
{
   bool chkd = CheckMailBox->Checked;
   Minutes->Enabled = chkd;
   UpDown1->Enabled = chkd;
   Label1->Enabled = chkd;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::ClearBtnClick(TObject *Sender)
{
   if (MsgBox(this, "Clear the passphrase cache, now?   ",
	  MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  ClearBtn->Enabled = false;
	  PGPCache->Clear();
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/options.htm>optshelp");
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::NNTPEditExit(TObject *Sender)
{
   int timeout = NNTPEdit->Text.ToIntDef(0);
   if (timeout != NNTPTimeOut->Position)
   {
	  if (timeout < NNTPTimeOut->Min)
	  {
		 MsgBox(this, "NNTP timeout must be at least " +
			AnsiString(NNTPTimeOut->Min) + " seconds!",
			MT_INFO, MB_OK);
		 PageControl->ActivePage = TabSheet3;
		 NNTPEdit->Text = NNTPTimeOut->Min;
		 ActiveControl = NNTPEdit;
		 return;
	  }
	  if (timeout > NNTPTimeOut->Max)
	  {
		 MsgBox(this, "NNTP timeout can be no greater than " +
			AnsiString(NNTPTimeOut->Max) + " seconds!",
			MT_INFO, MB_OK);
		 PageControl->ActivePage = TabSheet3;
		 NNTPEdit->Text = NNTPTimeOut->Max;
		 ActiveControl = NNTPEdit;
		 return;
	  }
	  NNTPTimeOut->Position = (short)timeout;
   }
}
//---------------------------------------------------------------------------

