/*
*  proxydlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "proxydlgs.h"
#include "global.h"
#include "utility.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TProxyDlg *ProxyDlg;
//---------------------------------------------------------------------------

bool __fastcall NewProxy(TForm *Owner, TStrings *Proxy, TStrings *Proxies)
{
   assert(Owner != NULL);

   TProxyDlg *proxydlg = new TProxyDlg(Owner, Proxy, Proxies);
   proxydlg->Caption = "New Proxy";
   proxydlg->ShowModal();
   bool ok = proxydlg->ModalResult == mrOk;
   delete proxydlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall EditProxy(TForm *Owner, TStrings *Proxy, TStrings *Proxies)
{
   assert(Owner != NULL);

   TProxyDlg *proxydlg = new TProxyDlg(Owner, Proxy, Proxies);
   proxydlg->Caption = "Editing Proxy";
   proxydlg->ShowModal();
   bool ok = proxydlg->ModalResult == mrOk;
   delete proxydlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TProxyDlg::TProxyDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TProxyDlg::TProxyDlg(TComponent* Owner, TStrings *Proxy,
   TStrings *Proxies) : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Host->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   NNTPPort->Font->Assign(Host->Font);
   Socks->Font->Assign(Host->Font);
   UserID->Font->Assign(Host->Font);
   Password->Font->Assign(Host->Font);
   delete ini;

   newproxy = NewProxy;
   proxies = Proxies;
   proxy = Proxy;

   if (proxy->Count)
   {
	  assert(proxy->Count);
	  Name->Text = ReadString(proxy, "proxy", "");
	  orgname = Name->Text;

	  Host->Text = ReadString(proxy, "host", "");
	  NNTPPort->Text = ReadString(proxy, "port", "");
	  TOR->Checked = ReadBool(proxy, "tor", false);
	  Socks->ItemIndex = Socks->Items->IndexOf(ReadString(proxy, "socks", "no"));
	  if (Socks->ItemIndex < 0)
		 Socks->ItemIndex = 0;
	  Auth->Checked = ReadBool(proxy, "auth", false);
	  UserID->Text = ReadString(proxy, "userid", "");
	  Password->Text = ReadString(proxy, "password", "");
	  AuthClick(Auth);
   }
   if (Name->Text.IsEmpty())
	  Name->Text = GetNextDefaultName();
   Name->Modified = false;
}
//---------------------------------------------------------------------------

__fastcall TProxyDlg::~TProxyDlg()
{
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
	  if (orgname != "" && Name->Text != orgname &&
		  proxies->IndexOfName(Name->Text) >= 0)
	  {
		 MsgBox(this,
			"The name you've chosen for this proxy, " + Name->Text + ",  \n"
			"already exists!",
			 MT_INFO, MB_OK);
		 CanClose = false;
		 return;
	  }

	  if (Host->Text.IsEmpty())
	  {
		 MsgBox(this,
		   "Sorry, Host is a required field!   ",
			MT_INFO, MB_OK);
		 ActiveControl = Host;
		 CanClose = false;
		 return;
	  }

	  if (NNTPPort->Text.IsEmpty())
	  {
		 MsgBox(this,
		   "An NNTP port number is required!   ",
			MT_INFO, MB_OK);
		 ActiveControl = NNTPPort;
		 CanClose = false;
		 return;
	  }

	  if (Socks->Text == "no")
	  {
		 MsgBox(this,
		   "NNTP requires Socks Level of 4a!   ",
			MT_INFO, MB_OK);
		 ActiveControl = Socks;
		 CanClose = false;
		 return;
	  }

	  if (Auth->Checked && (UserID->Text.IsEmpty() || Password->Text.IsEmpty()))
	  {
		 MsgBox(this,
			"Since you've enabled Authentication, both User ID and Password "
			"are required.",
			 MT_INFO, MB_OK);
		 ActiveControl = UserID->Text.IsEmpty() ? UserID : Password;
		 CanClose = false;
		 return;
	  }

	  if (Host->Text == "127.0.0.1" && NNTPPort->Text == "9050" && !TOR->Checked)
	  {
		 if (MsgBox(this,
			"Your host is '127.0.0.1' and your NNTP port is '9050'.  \n"
			"Is this a TOR proxy?",
			MT_CONFIRM, MB_YESNO) == IDYES)
		 {
			TOR->Checked = true;
			Socks->ItemIndex = 2;
		 }
	  }

	  if (TOR->Checked && Socks->Text == "no")
	  {
		  MsgBox(this,
			"You've indicated this is a TOR proxy but you have Socks set\n"
			"to 'no'. TOR is a socks proxy. You have to select a Socks level\n"
			"or untick the TOR checkbox. If the proxy is TOR you should set\n"
			"Socks to 4a.",
			 MT_CONFIRM, MB_OK);
		 ActiveControl = Socks;
		 CanClose = false;
		 return;
	  }

	  if (TOR->Checked && Socks->Text != "4a")
	  {
		 switch(MsgBox(this,
			"You've indicated this is a TOR proxy but you haven't chosen  \n"
			"Socks Level 4a. TOR requires Socks 4a. Shall I correct that?   ",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL:
			   ActiveControl = Socks;
			   CanClose = false;
			   return;
			case IDYES:
			   Socks->ItemIndex = Socks->Items->IndexOf("4a");
		 }
	  }
	  /*if (TOR->Checked && NNTPPort->Text != "9050")
	  {
		 switch(MsgBox(this,
			"You've indicated this is a TOR proxy but you haven't chosen port   \n"
			"9050. That's the port TOR normally uses. Shall I correct that?   ",
			 MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL:
			   ActiveControl = NNTPPort;
			   CanClose = false;
			   return;
			case IDYES:
			   NNTPPort->Text = "9050";
		 }
	  }*/
   }

   if (Modified())
   {
	  if (ModalResult == mrCancel)
	  {
		 switch (MsgBox(this,
		   "Your proxy info is modified. Save your changes? ",
			MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL:
			   CanClose = false;
			case IDNO:
			   return;
		 }
	  }

	  proxy->Values["proxy"] = Name->Text;
	  proxy->Values["host"] = Host->Text;
	  proxy->Values["port"] = NNTPPort->Text;
	  proxy->Values["socks"] = Socks->Text;
	  proxy->Values["tor"] = (int)TOR->Checked;
	  proxy->Values["auth"] = (int)Auth->Checked;
	  proxy->Values["userid"] = UserID->Text;
	  proxy->Values["password"] = Password->Text;
   }
   else
	  ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::AuthClick(TObject *Sender)
{
   if (Auth->Checked)
   {
      UserID->Color = clWindow;
      UserID->Enabled = true;
      Password->Color = clWindow;
      Password->Enabled = true;
      HideBox->Enabled = true;
      Label5->Enabled = true;
      Label6->Enabled = true;
   }
   else
   {
      UserID->ParentColor = true;
      UserID->Enabled = false;
      Password->ParentColor = true;
      Password->Enabled = false;
      HideBox->Checked = true;
      //Password->PasswordChar = '*';
      HideBox->Enabled = false;
      Label5->Enabled = false;
      Label6->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::HideBoxClick(TObject *Sender)
{
   if (!HideBox->Checked)
      Password->PasswordChar = '\0';
   else
      Password->PasswordChar = '*';
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::TORClick(TObject *Sender)
{
   if (TOR->Checked)
      Socks->ItemIndex = 2;
}
//---------------------------------------------------------------------------

bool __fastcall TProxyDlg::Modified()
{
   return Name->Modified ||
      Host->Text != ReadString(proxy, "host", "") ||
	  NNTPPort->Text != ReadString(proxy, "port", "") ||
	  Socks->ItemIndex != Socks->Items->IndexOf(ReadString(proxy, "socks", "no")) ||
	  TOR->Checked != ReadBool(proxy, "tor", false) ||
	  Auth->Checked != ReadBool(proxy, "auth", false) ||
	  UserID->Text != ReadString(proxy, "userid", "") ||
	  Password->Text != ReadString(proxy, "password", "");
}
//---------------------------------------------------------------------------

AnsiString __fastcall TProxyDlg::GetNextDefaultName()
{
   int i = 1;
   while (proxies->IndexOfName("proxy"+IntToStr(i)) >= 0)
	  i++;

   return "proxy"+IntToStr(i);
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/proxyhost.htm>proxyhelp");
}
//---------------------------------------------------------------------------

