/*
*  qsadbs.h 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef qsadbsH
#define qsadbsH
//---------------------------------------------------------------------------
#include "secdbs.h"
class TQSADB;
//---------------------------------------------------------------------------

TQSADB* __fastcall OpenQSADB(const AnsiString &FileName); //, bool InMemory=true);
bool __fastcall CloseQSADB(TQSADB *DB);

class TQSADB : public TSecureDB
{
public:
   __fastcall virtual TQSADB(const AnsiString &FileName);
   __fastcall virtual~TQSADB();
   // messages
   bool __fastcall LoadMessageIndex(TStrings *MsgIndex);
   bool __fastcall LoadMessage(const AnsiString &MsgKey, TStrings *Msg);
   bool __fastcall SaveMessage(const AnsiString &MsgKey, TStrings *Msg);
   bool __fastcall DelMessage(const AnsiString &MsgKey);
   // hosts
   bool __fastcall LoadAccounts(TStrings *accounts);
   bool __fastcall SaveAccounts(TStrings *accounts);
   bool __fastcall LoadAccount(const AnsiString &name, TStrings *account);
   bool __fastcall SaveAccount(const AnsiString &name, TStrings *account);
   bool __fastcall AccountExists(const AnsiString &name);
   bool __fastcall DelAccount(const AnsiString &name);
   // proxies
   bool __fastcall LoadProxies(TStrings *servers);
   bool __fastcall SaveProxies(TStrings *servers);
   bool __fastcall LoadProxy(const AnsiString &name, TStrings *proxy);
   bool __fastcall SaveProxy(const AnsiString &name, TStrings *proxy);
   bool __fastcall ProxyExists(const AnsiString &name);
   bool __fastcall DelProxy(const AnsiString &name);
   // subjects
   bool __fastcall LoadSubjects(TStrings *subjects);
   bool __fastcall SaveSubjects(TStrings *subjects);
   AnsiString __fastcall LoadSubject(const AnsiString &subno);
   bool __fastcall LoadSubject(const AnsiString &name, TStrings *subject);
   bool __fastcall SaveSubject(const AnsiString &name, TStrings *subject);
   bool __fastcall SubjectExists(const AnsiString &subject);
   AnsiString __fastcall GetSubjectItem(const AnsiString &subno, const AnsiString &item,
      const AnsiString &Default);
   bool __fastcall DelSubject(const AnsiString &subject);
   // nymservers
   bool __fastcall LoadNymservers(TStrings *servers);
   AnsiString __fastcall GetNymserverType(const AnsiString &server);
   bool __fastcall LoadNymserverDomains(TStrings *domains);
   bool __fastcall LoadNymserver(const AnsiString &name, TStrings *server);
   bool __fastcall SaveNymserver(const AnsiString &name, TStrings *server);
   bool __fastcall SaveNymservers(TStrings *servers);
   // status
   bool __fastcall SaveStatus(TStrings *Status);
   bool __fastcall LoadStatus(TStrings *Status);
};

const AnsiString NNTPLog    = "nntp";
const AnsiString Proxies    = "proxies";
const AnsiString NNTPProxy  = "nntp";
const AnsiString Hosts      = "hosts";
const AnsiString NNTPHosts  = "nntphosts";

// message read flag
const int mflRead	  		= 1; // unread = 0

AnsiString __fastcall Base64EncodeData(char *buffer, int length);
//---------------------------------------------------------------------------
extern PACKAGE TQSADB *DB;
//---------------------------------------------------------------------------
#endif
