/*
*  subjectsmandlgs.cpp 
*
*  Copyright (C) 2012 Richard Christman 
*
*  This file is part of QuickSilver AAM.
*
*  QuickSilver AAM is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver AAM, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver AAM grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "subjectsmandlgs.h"
#include "subjectsdlgs.h"
#include "global.h"
#include "utility.h"
#include "qsadbs.h"
#include "help.h"
#include "gpg4.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSubjectsManDlg *SubjectsManDlg;
int __fastcall SortBySubject(TStringList *List, int Index1, int Index2);
//---------------------------------------------------------------------------

bool __fastcall ManageSubjectLines(TForm *Owner)
{
   TSubjectsManDlg *subsdlg = new TSubjectsManDlg(Owner);
   subsdlg->ShowModal();
   delete subsdlg;
   return true;
}
//---------------------------------------------------------------------------

__fastcall TSubjectsManDlg::TSubjectsManDlg(TComponent* Owner)
	: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "subjectsdlg", ListView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height) +
			IntToStr(ListView->Columns->Items[0]->Width);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont",
	  OptsDefWindowFont));
   newsgroup = ini->ReadString("settings", "newsgroup", "alt.anonymous.messages");
   Caption = "AAM Subject Lines -- " + newsgroup;
   delete ini;

   subjects = new TStringList();
   DB->GetText("subjects", subjects);
   subjects->CustomSort(SortBySubject);
//look(subjects);
   TStringList *sub = new TStringList();
   for (int i = 0; i < subjects->Count; i++)
   {
	  sub->CommaText = subjects->Strings[i];
	  AnsiString userid = GetTok(0, GPGGetPublicKeyStr(sub->Values["keyid"]), ':');
	  sub->Add("userid="+GetEAddress(userid));
	  TListItem *item = ListView->Items->Add();
	  item->Caption = sub->Values["subject"];
	  item->Checked = sub->Values["active"] == "1";
	  item->SubItems->Add((sub->Values["esub"] == "1" ? "encrypted" : "hashed"));
   }
   delete sub;
   ListView->DoubleBuffered = true;
}
//---------------------------------------------------------------------------

__fastcall TSubjectsManDlg::~TSubjectsManDlg()
{
   if (IntToStr(Width)+IntToStr(Height)+
	   IntToStr(ListView->Column[0]->Width) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "subjectsdlg", ListView, ini);
	  delete ini;
   }
   delete subjects;
}
//---------------------------------------------------------------------------

int __fastcall SortBySubject(TStringList *List, int Index1, int Index2)
{
   AnsiString subject1, subject2;
   TStringList *data = new TStringList();
   data->CommaText = List->Strings[Index1];
   subject1 = data->Values["subject"];
   data->CommaText = List->Strings[Index2];
   subject2 = data->Values["subject"];
   delete data;
   return subject1.AnsiCompare(subject2);
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   TStringList *data = new TStringList();
   for (int i = 0; i < ListView->Items->Count; i++)
   {
	  TListItem *item = ListView->Items->Item[i];
	  AnsiString subandpas = subjects->Strings[item->Index];
	  data->CommaText = subjects->Strings[item->Index];
	  bool active = data->Values["active"] == "1";
	  if (item->Checked != active)
	  {
		 data->Values["active"] = item->Checked ? "1":"0";
		 subjects->Strings[item->Index] = data->CommaText;
		 modified = true;
	  }
   }
   delete data;

   if (modified)
	  DB->PutText("subjects", subjects);
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::PopupMenuPopup(TObject *Sender)
{
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::ListViewClick(TObject *Sender)
{
   DeleteBtn->Enabled = ListView->Selected;
   EditBtn->Enabled = DeleteBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::AddItemClick(TObject *Sender)
{
   AnsiString subandpas;
   if (CreateSubjectPassphrasePair(this, subandpas, subjects))
   {
	  TStringList *data = new TStringList();
	  data->CommaText = subandpas;
	  TListItem *item = ListView->Items->Add();
	  item->Caption = data->Values["subject"];
	  item->Checked = data->Values["active"] == "1";
	  item->SubItems->Add(data->Values["esub"] == "1" ? "encrypted" : "hashed");
	  subjects->Add(subandpas);
	  modified = true;
	  delete data;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::EditItemClick(TObject *Sender)
{
   assert(ListView->Selected);

   int idx = ListView->Selected->Index;
   AnsiString subandpas = subjects->Strings[idx];
   assert(subandpas != "");

   if (EditSubjectPassphrasePair(this, subandpas, subjects))
   {
	  TStringList *data = new TStringList();
	  data->CommaText = subandpas;
	  ListView->Selected->Caption = data->Values["subject"];
	  ListView->Selected->Checked = data->Values["active"] == "1";
	  ListView->Selected->SubItems->Strings[0] = data->Values["esub"] == "1" ? "encrypted" : "hashed";
	  subjects->Strings[idx] = subandpas;
	  modified = true;
	  delete data;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::DeleteItemClick(TObject *Sender)
{
   assert(ListView->Selected);

   if (MsgBox(this,
	  "Delete this subject/passphrase now?   ",
	   MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		  return;

   TStringList *data = new TStringList();
   AnsiString delitemsubject = ListView->Selected->Caption;
   for (int i = 0; i < subjects->Count; i++)
   {
	  data->CommaText = subjects->Strings[i];
	  if (data->Values["subject"] == delitemsubject)
	  {
		 subjects->Delete(i);
		 ListView->Selected->Delete();
		 modified = true;
		 break;
	  }
   }
   delete data;
}
//---------------------------------------------------------------------------

void __fastcall TSubjectsManDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/subjectman.htm>newshelp");
}
//---------------------------------------------------------------------------

